import tkinter as tk
import threading
import random

# ソートを行う関数（選択法で体重の重い（大きい）順にソートする）
def do_sort():
  # グローバル変数の宣言
  global DATA_NUM, event
  update_display(0, 0, "", "", "選択法で体重の重い順にソートします。")
  # 外側のループカウンタは、決定する位置の添字
  for dec_idx in range(0, DATA_NUM-1, 1):
    # 先頭の要素を仮の最大値に設定する
    max_idx = dec_idx
    update_display(dec_idx, dec_idx+1, "★決定位置", "▼比較位置",\
      f"★決定位置の{lst_data[max_idx][2]}を仮の最大値（太枠で示す）とします。", max_idx)
    # 内側のループカウンタは、比較するデータの位置の添字
    for cmp_idx in range(dec_idx + 1, DATA_NUM, 1):
      # 仮の最大値を更新する
      update_display(dec_idx, cmp_idx, "★決定位置", "▼比較位置",\
        f"{lst_data[max_idx][2]}と{lst_data[cmp_idx][2]}を比較します。", max_idx)
      if lst_data[max_idx][1] < lst_data[cmp_idx][1]:
        update_display(dec_idx, cmp_idx, "★決定位置", "▼比較位置",\
          f"{lst_data[max_idx][2]}よりも{lst_data[cmp_idx][2]}の方が重いので、"\
          "仮の最大値を更新します。", max_idx)
        max_idx = cmp_idx
      else:
        update_display(dec_idx, cmp_idx, "★決定位置", "▼比較位置",\
          f"{lst_data[max_idx][2]}よりも{lst_data[cmp_idx][2]}の方が重くないので、"\
          "仮の最大値を更新しません。", max_idx)
    # 最大値が確定したので、先頭の要素と最大値を交換する
    if dec_idx != max_idx:
      update_display(dec_idx, cmp_idx, "★決定位置", "▼比較位置",\
        f"{lst_data[max_idx][2]}が最大値として確定したので、★決定位置にいる"\
        f"{lst_data[dec_idx][2]}と交換します。", max_idx)
      temp = lst_data[dec_idx]
      lst_data[dec_idx] = lst_data[max_idx]
      lst_data[max_idx] = temp
    else:
      update_display(dec_idx, cmp_idx, "★決定位置", "▼比較位置",\
        f"★決定位置にいる{lst_data[dec_idx][2]}が最大値として確定しました。", dec_idx)
    # ソートが完了していない場合
    if dec_idx < DATA_NUM - 2:
      update_display(dec_idx, cmp_idx, "★決定位置", "▼比較位置",\
        "★決定位置を1つ先に進め、その1つ先を▼比較位置にして、処理を繰り返します。", dec_idx)
  # ソートが完了している場合
  update_display(dec_idx, cmp_idx, "★決定位置", "▼比較位置", "ソートが完了しました。")

# 「次の動作」ボタンがクリックされたときに呼び出される関数
def btn_next_click():
  # グローバル変数の宣言
  global event
  # イベントを発生させ、待機しているスレッドを再開する
  event.set()

# 画面の表示を更新する関数
# lpc_outer     外側のループカウンタ
# lpc_inner     内側のループカウンタ
# outer_text    外側のループカウンタの位置のラベルに表示する文字列
# inner_text    内側のループカウンタの位置のラベルに表示する文字列
# exp_text      説明のラベルに表示する文字列
# *solid_idx    太枠を表示する画像の添字（可変長引数）
def update_display(lpc_outer, lpc_inner, outer_text, inner_text, exp_text, *solid_idx):
  # グローバル変数の宣言
  global DATA_NUM, lbl_lpc_outer, lbl_image, lbl_lpc_inner, lst_data,\
    lbl_explanation, event
  # すべてのラベルの表示を初期状態にする
  for idx in range(DATA_NUM):
    lbl_lpc_outer[idx]["text"] = ""
    lbl_lpc_inner[idx]["text"] = ""
  # 外側のループカウンタの位置のラベルを設定する
  lbl_lpc_outer[lpc_outer]["text"] = outer_text
  # 内側のループカウンタの位置のラベルを設定する
  lbl_lpc_inner[lpc_inner]["text"] = inner_text
  # すべてのラベルに画像を表示する
  for idx in range(DATA_NUM):
    lbl_image[idx]["image"] = lst_data[idx][0]
    # solid_idxで指定された画像に太枠を表示する
    if idx in solid_idx:
      lbl_image[idx]["relief"] = "solid"
    else:
      lbl_image[idx]["relief"] = "ridge"
  # 説明のラベルを設定する
  lbl_explanation["text"] = exp_text
  # イベントが発生するまで、スレッドを待機させる
  event.wait()
  # イベントをクリアして、スレッドが待機できるようにする
  event.clear()

# ソート対象のデータの数
DATA_NUM = 5

# Tkのルートを作成する
root = tk.Tk()

# ウィンドウにタイトルを設定する
root.title("身近なソートのアルゴリズム")

# 上段、中段、下段のフレームを作成する
frm_upper = tk.LabelFrame(root, relief="flat")
frm_upper.grid(row=0, column=0, sticky=tk.W)
frm_middle = tk.LabelFrame(root, relief="flat")
frm_middle.grid(row=1, column=0, sticky=tk.W)
frm_lower = tk.LabelFrame(root, relief="flat")
frm_lower.grid(row=2, column=0, sticky=tk.W)

# 外側と内側のループカウンタの位置を示すラベルを作成する
lbl_lpc_outer = []
lbl_lpc_inner = []
for idx in range(DATA_NUM):
  lbl_lpc_outer.append(tk.Label(frm_upper, text="", background="#ffff00",\
    relief="ridge", borderwidth=3))
  lbl_lpc_outer[idx].grid(row=0, column=idx, padx=5, pady=1,\
    sticky=tk.W + tk.E + tk.N + tk.S)
  lbl_lpc_inner.append(tk.Label(frm_upper, text="", background="#00ffff",\
    relief="ridge", borderwidth=3))
  lbl_lpc_inner[idx].grid(row=1, column=idx, padx=5, pady=1,\
    sticky=tk.W + tk.E + tk.N + tk.S)

# ソート対象のデータ（画像、値、呼び名）（選択法）
lst_data = [
  (tk.PhotoImage(file="elephant.png"), 5000, "ゾウ"),
  (tk.PhotoImage(file="horse.png"), 500, "ウマ"),
  (tk.PhotoImage(file="dog.png"), 10, "イヌ"),
  (tk.PhotoImage(file="cat.png"), 5, "ネコ"),
  (tk.PhotoImage(file="mouse.png"), 0.5, "ネズミ")
]

# ソート対象のデータをシャッフルする
random.shuffle(lst_data)

# 画像を表示するラベルを作成する
lbl_image = []
for idx in range(DATA_NUM):
  lbl_image.append(tk.Label(frm_upper, image=lst_data[idx][0],\
    relief="ridge", borderwidth=3))
  lbl_image[idx].grid(row=2, column=idx, padx=5, pady=5,\
    sticky=tk.W + tk.E + tk.N + tk.S)

# 説明を表示するラベルを作成する
lbl_explanation = tk.Label(frm_middle,\
  text="「次の動作」ボタンをクリックしてください。",\
  background="#ffffff", relief="ridge", borderwidth=3)
lbl_explanation.grid(row=0, column=0, padx=5, pady=5, sticky=tk.W)

# 「次の動作」ボタンを作成する
btn_next = tk.Button(frm_lower, text="次の動作", command=btn_next_click)
btn_next.grid(row=0, column=0, padx=5, pady=5)

# Eventオブジェクトを作成する
# （イベントが発生するまでスレッドを待機させる機能を持つ）
event = threading.Event()

# ソートを行う関数をスレッドにする
thread = threading.Thread(target=do_sort)

# スレッドをデーモン化する（親スレッドが終了すると子スレッドも終了する）
thread.setDaemon(True)

# スレッドを開始する
thread.start()

# イベント待ちの無限ループ
root.mainloop()
